#!/usr/local/bin/perl
#!~PERL~
# $Revision: 1.5 $$Date: 2001/08/07 16:55:54 $

use 5.6.0;
use strict;
use warnings;
use Class::Struct qw(struct);

my $argv0 = $0;
$argv0 =~ s{.*/}{};

struct 'range' => [
	dir => '$',    # -1:'<', 0:'|', 1:'>', 2:n/a
	x0 => '$',
	x1 => '$',
	name => '$',
];

struct 'element' => [
	gene => 'range',
	cds => 'range',
	exons => '@',
];

sub cmp_range
{
	return $_[0]->x0 - $_[1]->x0;
}

sub cmp_element 
{
	return $_[0]->gene->x0 - $_[1]->gene->x0;
}

sub emit_range 
{
	my $p = shift;
	my @d = ("< ", "| ", "> ", "");
	print $d[$p->dir+1], $p->x0, " ", $p->x1;
	if ($p->name) { print " ", $p->name; }
	print "\n";
}

sub emit_element
{
	my $x = shift;
	emit_range $x->gene;
	if (defined $x->cds->dir) { print "+ "; emit_range $x->cds; }
	foreach my $p (@{$x->exons}) { emit_range $p; }
	print "\n";
}

$|=1;
my @stuff = ();
my $header = "";
my $e = undef;
my $narg = @ARGV;
if ($narg == 0) {
	print STDERR "Usage:  $argv0 exons_file > output_file\n";
	exit 1;
}
while (<>) {
	chomp;
	next if /^\s*#/;
	next if /^\s*$/;

	my $x = undef;
	if ( /^\s*[a-zA-Z]/ ) {
		$header = $_ unless $header;
		next;
	} elsif ( /^\s*([<|>])\s*(\d+)\s+(\d+)\s*(.*)$/ ) {
		if ($e) { push @stuff, $e; }
		my $d = ($1 eq "<") ? -1 : (($1 eq ">") ? 1 : 0);
		$x = new range(dir=>$d, x0=>$2, x1=>$3, name=>"$4");
		$e = new element(gene=>undef, cds=>undef, exons=>undef);
		$e->gene($x);
		$e->cds(new range(dir=>undef, x0=>0, x1=>0, name=>""));
	} elsif ( /^\s*(\+)\s*(\d+)\s+(\d+)\s*(.*)$/ ) {
		die unless $e;
		$x = new range(dir=>2, x0=>$2, x1=>$3, name=>"$4");
		$e->cds($x);
	} elsif ( /^\s*(\d+)\s+(\d+)\s*(.*)$/ ) {
		die unless $e;
		$x = new range(dir=>2, x0=>$1, x1=>$2, name=>"$3");
		push @{$e->exons}, $x;
	} else {
		print "# foo? $_\n";
	}
}
if ($e) { push @stuff, $e; }

print $header, "\n\n";
foreach $e (sort { cmp_element($a, $b) } @stuff) {
	@{$e->exons} = sort { cmp_range($a, $b) } @{$e->exons};
	emit_element($e);
}

